@@ -2,16 +2,16 @@ use std::io::{self, Write};
 use std::path::PathBuf;
 use clap::{Command, Arg, ArgMatches};
 
-use sphinx_lang;
 use sphinx_lang::frontend;
 use sphinx_lang::BuildErrors;
 use sphinx_lang::source::{ModuleSource, SourceText};
+use sphinx_lang::parser::expr::Expr;
 use sphinx_lang::parser::stmt::{Stmt, StmtMeta};
 use sphinx_lang::codegen::{Program, CompiledProgram};
 use sphinx_lang::runtime::VirtualMachine;
 use sphinx_lang::runtime::module::{ModuleCache, GlobalEnv};
 use sphinx_lang::runtime::strings::StringInterner;
-use sphinx_lang::debug::symbol::BufferedResolver;
+use sphinx_lang::debug::symbol::resolver::BufferedResolver;
 
 fn main() {
     env_logger::init();
@@ -46,6 +46,11 @@ fn main() {
             Arg::new("compile_only")
             .short('d')
             .help("Produce compiled bytecode instead of executing (not implemented)")
+        )
+        .arg(
+            Arg::new("debug")
+            .long("debug")
+            .help("Enable step-through debugging")
         );
     
     let version = app.get_version().unwrap();
@@ -81,55 +86,79 @@ fn main() {
             let mut module_cache = ModuleCache::new();
             let module_id = module_cache.insert(program.data, Some(name.to_string()), Some(source));
             
-            let repl_env = GlobalEnv::new();
-            
-            let vm = VirtualMachine::new_repl(&module_cache, &repl_env, module_id, &program.main);
-            vm.run().expect("runtime error");
-            
-            println!("\nSphinx Version {}\n", version);
-            Repl::new(&mut module_cache, &repl_env).run()
-        }
-    }
-    else {
-        if let Some(build) = build_program(&args, name, &source) {
-            let program = Program::load(build.program);
-            
-            let mut module_cache = ModuleCache::new();
-            let module_id = module_cache.insert(program.data, Some(name.to_string()), Some(source));
             
             let vm = VirtualMachine::new(&module_cache, module_id, &program.main);
             vm.run().expect("runtime error");
+            
+            println!("\nSphinx Version {}\n", version);
+            
+            let repl_env = module_cache.get(&module_id).unwrap().globals().clone();
+            Repl::new(&mut module_cache, &repl_env).run()
+        }
+    }
+    else if let Some(build) = build_program(&args, name, &source) {
+        let program = Program::load(build.program);
+        
+        let mut module_cache = ModuleCache::new();
+        let module_id = module_cache.insert(program.data, Some(name.to_string()), Some(source));
+        
+        let vm = VirtualMachine::new(&module_cache, module_id, &program.main);
+        if args.is_present("debug") {
+            run_debugger(vm);
+        } else {
+            vm.run().expect("runtime error");
         }
     }
 }
 
 
-fn build_program<'m>(_args: &ArgMatches, name: &str, source: &ModuleSource) -> Option<CompiledProgram> {
+fn build_program(_args: &ArgMatches, name: &str, source: &ModuleSource) -> Option<CompiledProgram> {
     // build module
-    let build_result = sphinx_lang::build_module(source);
-    if build_result.is_err() {
-        match build_result.unwrap_err() {
-            BuildErrors::Source(error) => {
-                println!("Error reading source: {}.", error);
+    match sphinx_lang::build_module(source) {
+        Err(error) => {
+            match error {
+                BuildErrors::Source(error) => {
+                    println!("Error reading source: {}.", error);
+                }
+                
+                BuildErrors::Syntax(errors) => {
+                    println!("Errors in file \"{}\":\n", name);
+                    frontend::print_source_errors(source, &errors);
+                }
+                
+                BuildErrors::Compile(errors) => {
+                    println!("Errors in file \"{}\":\n", name);
+                    frontend::print_source_errors(source, &errors);
+                }
             }
             
-            BuildErrors::Syntax(errors) => {
-                println!("Errors in file \"{}\":\n", name);
-                frontend::print_source_errors(source, &errors);
+            None
+        },
+        
+        Ok(program) => Some(program)
+    }
+}
+
+fn run_debugger(vm: VirtualMachine) {
+    for status in vm.run_steps() {
+        match status {
+            Err(error) => {
+                println!("Runtime error: {:?}", error);
+                break;
             }
-            
-            BuildErrors::Compile(errors) => {
-                println!("Errors in file \"{}\":\n", name);
-                frontend::print_source_errors(source, &errors);
+            Ok(snapshot) => {
+                println!("{}", snapshot);
             }
         }
-        return None;
+        
+        let mut input = String::new();
+        let result = io::stdin().read_line(&mut input);
+        if let Err(..) = result { }
     }
     
-    return Some(build_result.unwrap())
+    println!("Execution stopped.");
 }
 
-
 fn parse_and_print_ast(_args: &ArgMatches, name: &str, source: &ModuleSource) {
     let source_text = match source.read_text() {
         Ok(source_text) => source_text,
@@ -179,7 +208,7 @@ impl<'m> Repl<'m> {
     }
     
     fn read_line(&self, prompt: &'static str) -> ReadLine {
-        io::stdout().write(prompt.as_bytes()).unwrap();
+        io::stdout().write_all(prompt.as_bytes()).unwrap();
         io::stdout().flush().unwrap();
         
         let mut input = String::new();
@@ -195,7 +224,7 @@ impl<'m> Repl<'m> {
             return ReadLine::Empty;
         }
         
-        if input == "quit" || input.chars().find(|c| *c == '\x04').is_some() {
+        if input == "quit" || input.chars().any(|c| c == '\x04') {
             return ReadLine::Quit;
         }
         
@@ -263,7 +292,7 @@ impl<'m> Repl<'m> {
             if let Some(stmt) = ast.pop() {
                 let (mut stmt, symbol) = stmt.take();
                 if let Stmt::Expression(expr) = stmt {
-                    stmt = Stmt::Echo(expr);
+                    stmt = Stmt::Expression(Expr::Echo(Box::new(expr)));
                 }
                 ast.push(StmtMeta::new(stmt, symbol))
             }
@@ -280,13 +309,17 @@ impl<'m> Repl<'m> {
             
             let program = Program::load(build.program);
             
-            let module_id = self.module_cache.insert(program.data, None, None);
+            let module_id = self.module_cache.insert_with_globals(program.data, None, None, self.repl_env.clone());
             
-            let vm = VirtualMachine::new_repl(self.module_cache, self.repl_env, module_id, &program.main);
+            let vm = VirtualMachine::repl(self.module_cache, self.repl_env, module_id, &program.main);
             if let Err(error) = vm.run() {
                 println!("Runtime error: {:?}", error);
             }
             
+            // This is super inefficient, but it's the REPL, so that's okay?
+            let module = self.module_cache.get(&module_id).unwrap();
+            self.repl_env.borrow_mut().extend(&module.globals().borrow());
+            
         }
         
     }
