@@ -4,7 +4,7 @@ use log;
 use std::iter;
 use std::mem;
 
-use crate::language::{IntType, FloatType};
+use crate::language::{IntType, FloatType, InternSymbol};
 use crate::parser::stmt::{StmtMeta, Stmt, Label, StmtList, ControlFlow};
 use crate::parser::expr::{Expr, ExprMeta, ExprBlock, ConditionalBranch};
 use crate::parser::primary::{Atom, Primary, AccessItem};
@@ -12,21 +12,23 @@ use crate::parser::lvalue::{Assignment, Declaration, LValue, DeclType};
 use crate::parser::fundefs::{FunctionDef, SignatureDef, ParamDef, DefaultDef};
 use crate::runtime::vm::LocalIndex;
 use crate::runtime::types::operator::{UnaryOp, BinaryOp, Arithmetic, Bitwise, Shift, Comparison, Logical};
-use crate::runtime::types::function::{Function, Signature, Parameter};
-use crate::runtime::strings::{StringInterner, InternSymbol};
-use crate::debug::DebugSymbol;
-use crate::debug::dasm::{ChunkSymbols, DebugSymbolsRLE};
+use crate::runtime::types::function::Function;
+use crate::runtime::strings::{StringInterner};
+use crate::debug::symbol::{DebugSymbol, ChunkSymbols, DebugSymbolTable};
 
 pub mod chunk;
+pub mod consts;
 pub mod opcodes;
 pub mod errors;
 
 pub use opcodes::OpCode;
-pub use chunk::{UnloadedProgram, Program, ProgramData, ChunkID, ConstID, Constant};
+pub use chunk::{UnloadedProgram, Program, ProgramData, ChunkID};
+pub use consts::{ConstID, Constant};
 pub use errors::{CompileResult, CompileError, ErrorKind};
 
 use opcodes::*;
 use chunk::{ChunkBuilder, ChunkInfo, ChunkBuf};
+use consts::{UnloadedSignature, UnloadedParam};
 
 
 // Helpers
@@ -92,14 +94,16 @@ const fn get_jump_opcode(jump: Jump, offset: JumpOffset) -> OpCode {
         (Jump::Uncond,  JumpOffset::Short(..)) => OpCode::Jump,
         (Jump::IfFalse, JumpOffset::Short(..)) => OpCode::JumpIfFalse,
         (Jump::IfTrue,  JumpOffset::Short(..)) => OpCode::JumpIfTrue,
-        (Jump::Uncond,  JumpOffset::Long(..))  => unimplemented!(),
-        (Jump::IfFalse, JumpOffset::Long(..))  => unimplemented!(),
-        (Jump::IfTrue,  JumpOffset::Long(..))  => unimplemented!(),
+        
+        (Jump::Uncond,  JumpOffset::Long(..))  => OpCode::LongJump,
+        (Jump::IfFalse, JumpOffset::Long(..))  => OpCode::LongJumpIfFalse,
+        (Jump::IfTrue,  JumpOffset::Long(..))  => OpCode::LongJumpIfTrue,
         
         (Jump::PopIfFalse, JumpOffset::Short(..))  => OpCode::PopJumpIfFalse,
         (Jump::PopIfTrue,  JumpOffset::Short(..))  => OpCode::PopJumpIfTrue,
-        (Jump::PopIfFalse, JumpOffset::Long(..))   => unimplemented!(),
-        (Jump::PopIfTrue,  JumpOffset::Long(..))   => unimplemented!(),
+        
+        (Jump::PopIfFalse, JumpOffset::Long(..))   => OpCode::PopLongJumpIfFalse,
+        (Jump::PopIfTrue,  JumpOffset::Long(..))   => OpCode::PopLongJumpIfTrue,
     }
 }
 
@@ -121,7 +125,7 @@ enum LocalName {
 struct Local {
     decl: DeclType,
     name: LocalName,
-    offset: LocalIndex,
+    index: LocalIndex,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -138,9 +142,11 @@ enum ScopeTag {
 struct Scope {
     tag: ScopeTag,
     depth: usize,
-    offset: Option<LocalIndex>,
-    locals: Vec<Local>,
+    chunk: Option<ChunkID>,
     symbol: Option<DebugSymbol>,
+    
+    frame: Option<LocalIndex>,
+    locals: Vec<Local>,
 }
 
 impl Scope {
@@ -148,8 +154,8 @@ impl Scope {
         self.symbol.as_ref()
     }
     
-    fn last_offset(&self) -> Option<LocalIndex> {
-        self.locals.last().map_or(self.offset, |local| Some(local.offset))
+    fn last_index(&self) -> Option<LocalIndex> {
+        self.locals.last().map_or(self.frame, |local| Some(local.index))
     }
     
     fn find_local(&self, name: &LocalName) -> Option<&Local> {
@@ -161,26 +167,21 @@ impl Scope {
     }
     
     fn push_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<&Local> {
-        let offset = self.last_offset().map_or(
+        let index = self.last_index().map_or(
             Ok(0),
-            |offset| offset.checked_add(1)
+            |index| index.checked_add(1)
                 .ok_or_else(|| CompileError::from(ErrorKind::LocalVariableLimit))
         )?;
         
         let local = Local {
-            decl, name, offset,
+            decl, name, index,
         };
         self.locals.push(local);
         Ok(self.locals.last().unwrap())
     }
     
-    // i.e., without the "nonlocal" keyword
-    // this returns true if we can assign into the enclosing scope
-    fn allow_enclosing_assignment(&self) -> bool {
-        match self.tag {
-            ScopeTag::Loop | ScopeTag::Branch => true,
-            _ => false,
-        }
+    fn is_frame_start(&self) -> bool {
+        self.frame.is_none()
     }
 }
 
@@ -206,14 +207,20 @@ impl ScopeTracker {
         self.scopes.last_mut()
     }
     
-    fn push_scope(&mut self, tag: ScopeTag, symbol: Option<&DebugSymbol>) {
-        let offset = self.local_scope().and_then(|scope| scope.last_offset());
+    fn push_scope(&mut self, tag: ScopeTag, chunk: Option<ChunkID>, symbol: Option<&DebugSymbol>) {
+        let frame;
+        if tag == ScopeTag::Function {
+            frame = None; // inside a function, locals are indexed from the start of the frame
+        } else {
+            frame = self.local_scope().and_then(|scope| scope.last_index());
+        }
         
         let scope = Scope {
             tag, 
-            symbol: symbol.map(|sym| *sym),
+            chunk,
+            symbol: symbol.copied(),
             depth: self.scopes.len(),
-            offset,
+            frame,
             locals: Vec::new(),
         };
         self.scopes.push(scope);
@@ -239,43 +246,23 @@ impl ScopeTracker {
         self.scopes.iter().rev()
     }
     
-    // find the nearest local in any scope
-    fn resolve_local(&self, name: &LocalName) -> Option<&Local> {
-        self.scopes.iter().rev().find_map(|scope| scope.find_local(name))
-    }
-    
     // find the nearest local in scopes that allow nonlocal assignment
-    fn resolve_local_strict(&self, name: &LocalName) -> Option<&Local> {
+    fn resolve_local(&self, name: &LocalName) -> Option<&Local> {
         for scope in self.iter_scopes() {
             let local = scope.find_local(name);
             if local.is_some() {
                 return local;
             }
-            
-            if !scope.allow_enclosing_assignment() {
+            if scope.is_frame_start() {
                 break;
             }
         }
-        
         None
     }
     
-    // search scopes that would have been skipped by resolve_local_strict()
-    fn resolve_nonlocal(&self, name: &LocalName) -> Option<&Local> {
-        let mut is_local = true;
-        for scope in self.iter_scopes() {
-            if is_local {
-                is_local &= scope.allow_enclosing_assignment();
-                continue;
-            }
-            
-            let local = scope.find_local(name);
-            if local.is_some() {
-                return local;
-            }
-        }
-        
-        None
+    // without the nonlocal keyword, that is
+    fn can_assign_global(&self) -> bool {
+        self.scopes.iter().skip(1).all(|scope| !scope.is_frame_start())
     }
 }
 
@@ -301,7 +288,7 @@ impl Compiler {
     pub fn new(strings: StringInterner) -> Self {
         // insert RLE container for main chunk
         let mut symbols = ChunkSymbols::new();
-        symbols.insert(None, DebugSymbolsRLE::new());
+        symbols.insert(None, DebugSymbolTable::new());
         
         Self {
             builder: ChunkBuilder::with_strings(strings),
@@ -313,9 +300,9 @@ impl Compiler {
     
     fn new_chunk(&mut self, info: ChunkInfo) -> CompileResult<ChunkID> {
         let chunk_id = self.builder.new_chunk(info)?;
-        if !self.symbols.contains_key(&Some(chunk_id)) {
-            self.symbols.insert(Some(chunk_id), DebugSymbolsRLE::new());
-        }
+        self.symbols.entry(Some(chunk_id))
+            .or_insert_with(DebugSymbolTable::new);
+        
         Ok(chunk_id)
     }
     
@@ -406,16 +393,17 @@ impl CodeGenerator<'_> {
         self.chunk().len()
     }
     
-    fn push_symbol(&mut self, symbol: Option<&DebugSymbol>) {
+    fn push_symbol(&mut self, symbol: DebugSymbol) {
         let chunk_id = self.chunk_id;
+        let offset = self.current_offset();
         self.symbols_mut()
             .get_mut(&chunk_id).unwrap()
-            .push(symbol.map(|symbol| *symbol))
+            .insert(offset, symbol)
     }
     
     fn create_chunk(&mut self, symbol: Option<&DebugSymbol>) -> CompileResult<CodeGenerator> {
         let info = ChunkInfo {
-            symbol: symbol.map(|symbol| *symbol),
+            symbol: symbol.copied(),
         };
         let chunk_id = self.compiler.new_chunk(info)?;
         Ok(self.compiler.get_chunk(Some(chunk_id)))
@@ -425,22 +413,34 @@ impl CodeGenerator<'_> {
     
     fn emit_instr(&mut self, symbol: Option<&DebugSymbol>, opcode: OpCode) {
         debug_assert!(opcode.instr_len() == 1);
-        self.push_symbol(symbol);
+        
+        if let Some(symbol) = symbol {
+            self.push_symbol(*symbol);
+        }
+        
         self.chunk_mut().push_byte(opcode);
     }
     
     fn emit_instr_byte(&mut self, symbol: Option<&DebugSymbol>, opcode: OpCode, byte: u8) {
         debug_assert!(opcode.instr_len() == 2);
-        self.push_symbol(symbol);
+        
+        if let Some(symbol) = symbol {
+            self.push_symbol(*symbol);
+        }
+        
         self.chunk_mut().push_byte(opcode);
         self.chunk_mut().push_byte(byte);
     }
     
     fn emit_instr_data(&mut self, symbol: Option<&DebugSymbol>, opcode: OpCode, bytes: &[u8]) {
         debug_assert!(opcode.instr_len() == 1 + bytes.len());
-        self.push_symbol(symbol);
+        
+        if let Some(symbol) = symbol {
+            self.push_symbol(*symbol);
+        }
+        
         self.chunk_mut().push_byte(opcode);
-        self.chunk_mut().extend_bytes(&bytes);
+        self.chunk_mut().extend_bytes(bytes);
     }
     
     ///////// Patching Bytecode /////////
@@ -453,7 +453,9 @@ impl CodeGenerator<'_> {
     }
     
     fn emit_dummy_instr(&mut self, symbol: Option<&DebugSymbol>, width: usize) {
-        self.push_symbol(symbol);
+        if let Some(symbol) = symbol {
+            self.push_symbol(*symbol);
+        }
         
         for i in 0..width {
             self.chunk_mut().push_byte(OpCode::Nop);
@@ -555,7 +557,8 @@ impl CodeGenerator<'_> {
     ///////// Scopes /////////
     
     fn emit_begin_scope(&mut self, tag: ScopeTag, symbol: Option<&DebugSymbol>) {
-        self.scope_mut().push_scope(tag, symbol);
+        let chunk_id = self.chunk_id;
+        self.scope_mut().push_scope(tag, chunk_id, symbol);
     }
     
     fn emit_end_scope(&mut self) {
@@ -576,14 +579,14 @@ impl CodeGenerator<'_> {
     
     // If the local name cannot be found, no instructions are emitted and None is returned
     fn try_emit_load_local(&mut self, symbol: Option<&DebugSymbol>, name: &LocalName) -> Option<u16> {
-        if let Some(offset) = self.scope().resolve_local(name).map(|local| local.offset) {
-            if let Ok(offset) = u8::try_from(offset) {
-                self.emit_instr_byte(symbol, OpCode::LoadLocal, offset);
+        if let Some(index) = self.scope().resolve_local(name).map(|local| local.index) {
+            if let Ok(index) = u8::try_from(index) {
+                self.emit_instr_byte(symbol, OpCode::LoadLocal, index);
             } else {
-                self.emit_instr_data(symbol, OpCode::LoadLocal16, &offset.to_le_bytes());
+                self.emit_instr_data(symbol, OpCode::LoadLocal16, &index.to_le_bytes());
             }
             
-            Some(offset)
+            Some(index)
         } else{
             None
         }
@@ -605,11 +608,6 @@ impl CodeGenerator<'_> {
             
             Stmt::ForLoop { } => unimplemented!(),
             
-            Stmt::Echo(expr) => {
-                self.compile_expr(symbol, expr)?;
-                self.emit_instr(symbol, OpCode::Inspect);
-                self.emit_instr(symbol, OpCode::Pop);
-            },
             Stmt::Assert(expr) => {
                 self.compile_expr(symbol, expr)?;
                 self.emit_instr(symbol, OpCode::Assert);
@@ -633,14 +631,20 @@ impl CodeGenerator<'_> {
         // handle control flow
         if let Some(control) = stmt_list.end_control() {
             match control {
-                ControlFlow::Continue(label) => {
+                ControlFlow::Continue(label, symbol) => {
                     unimplemented!()
                 }
-                ControlFlow::Break(label, expr) => {
+                ControlFlow::Break(label, expr, symbol) => {
                     unimplemented!()
                 }
-                ControlFlow::Return(expr) => {
-                    unimplemented!()
+                ControlFlow::Return(expr, symbol) => {
+                    let symbol = symbol.as_ref();
+                    if let Some(expr) = expr {
+                        self.compile_expr(symbol, expr)?;
+                    } else {
+                        self.emit_instr(symbol, OpCode::Nil);
+                    }
+                    self.emit_instr(symbol, OpCode::Return);
                 }
             }
         }
@@ -720,6 +724,11 @@ impl CodeGenerator<'_> {
             Expr::IfExpr { branches, else_clause } => self.compile_if_expression(symbol, branches, else_clause.as_ref().map(|expr| &**expr))?,
             
             Expr::FunctionDef(fundef) => self.compile_function_def(symbol, fundef)?,
+            
+            Expr::Echo(expr) => {
+                self.compile_expr(symbol, expr)?;
+                self.emit_instr(symbol, OpCode::Inspect);
+            },
         }
         Ok(())
     }
@@ -895,9 +904,9 @@ impl CodeGenerator<'_> {
     fn compile_declaration(&mut self, symbol: Option<&DebugSymbol>, decl: DeclarationRef) -> CompileResult<()> {
         match &decl.lhs {
             LValue::Identifier(name) => if self.scope().is_global_scope() {
-                self.compile_decl_global_name(symbol, decl.decl, *name, &decl.init)
+                self.compile_decl_global_name(symbol, decl.decl, *name, decl.init)
             } else {
-                self.compile_decl_local_name(symbol, decl.decl, *name, &decl.init)
+                self.compile_decl_local_name(symbol, decl.decl, *name, decl.init)
             },
             
             LValue::Attribute(target) => unimplemented!(),
@@ -967,7 +976,7 @@ impl CodeGenerator<'_> {
             LValue::Index(target) => unimplemented!(),
             
             LValue::Tuple(..) if assign.op.is_some() => {
-                return Err(CompileError::new("can't use update-assigment when assigning to a tuple"))
+                Err(CompileError::new("can't use update-assigment when assigning to a tuple"))
             },
             
             LValue::Tuple(target_list) => match assign.rhs {
@@ -1013,12 +1022,12 @@ impl CodeGenerator<'_> {
         if let Some(op) = assign.op {
             // update assignment
             self.compile_name_lookup(symbol, name)?;
-            self.compile_expr(symbol, &assign.rhs)?;
+            self.compile_expr(symbol, assign.rhs)?;
             self.emit_binary_op(symbol, &op);
             
         } else {
             // normal assignment
-            self.compile_expr(symbol, &assign.rhs)?;
+            self.compile_expr(symbol, assign.rhs)?;
             
         }
         
@@ -1028,39 +1037,40 @@ impl CodeGenerator<'_> {
             let local = LocalName::Symbol(*name);
             
             // check if the name is found in the local scope...
-            let result = self.scope().resolve_local_strict(&local).map(|local| (local.decl, local.offset));
+            let result = self.scope().resolve_local(&local).map(|local| (local.decl, local.index));
             
-            if let Some((decl, offset)) = result {
+            if let Some((decl, index)) = result {
                 if decl != DeclType::Mutable {
                     return Err(CompileError::from(ErrorKind::CantAssignImmutable));
                 }
-                self.emit_assign_local(symbol, offset);
+                self.emit_assign_local(symbol, index);
                 
                 return Ok(());
             }
             
+            // TODO upvalues
+            
             // otherwise search enclosing scopes...
+            // let result = self.scope().resolve_nonlocal(&local).map(|local| (local.decl, local.index));
             
-            let result = self.scope().resolve_nonlocal(&local).map(|local| (local.decl, local.offset));
-            
-            if let Some((decl, offset)) = result {
-                if !assign.nonlocal {
-                    return Err(CompileError::from(ErrorKind::CantAssignNonLocal));
-                }
-                if decl != DeclType::Mutable {
-                    return Err(CompileError::from(ErrorKind::CantAssignImmutable));
-                }
-                self.emit_assign_local(symbol, offset);
+            // if let Some((decl, index)) = result {
+            //     if !assign.nonlocal {
+            //         return Err(CompileError::from(ErrorKind::CantAssignNonLocal));
+            //     }
+            //     if decl != DeclType::Mutable {
+            //         return Err(CompileError::from(ErrorKind::CantAssignImmutable));
+            //     }
+            //     self.emit_assign_local(symbol, index);
                 
-                return Ok(());
-            }
+            //     return Ok(());
+            // }
             
         }
         
         // ...finally, try to assign global
         
         // allow assignment to global only if all enclosing scopes permit it
-        if !assign.nonlocal && !self.scope().iter_scopes().all(|scope| scope.allow_enclosing_assignment()) {
+        if !assign.nonlocal && !self.scope().can_assign_global() {
             return Err(CompileError::from(ErrorKind::CantAssignNonLocal));
         }
         
@@ -1169,7 +1179,7 @@ impl CodeGenerator<'_> {
         
         // and a new local scope
         // don't need to emit new scope instructions, should handled by function call
-        chunk.scope_mut().push_scope(ScopeTag::Function, symbol);
+        chunk.scope_mut().push_scope(ScopeTag::Function, Some(chunk_id), symbol);
         
         // don't need to generate IN_LOCAL instructions for these, the VM should include them automatically
         chunk.scope_mut().insert_local(DeclType::Immutable, LocalName::Receiver)?;
@@ -1179,9 +1189,7 @@ impl CodeGenerator<'_> {
         chunk.compile_function_preamble(symbol, fundef)?;
         
         // function body
-        for stmt in fundef.body.stmt_list().iter() {
-            chunk.push_stmt(stmt)?;
-        }
+        chunk.compile_stmt_list(fundef.body.stmt_list())?;
         
         // function result
         if let Some(expr) = fundef.body.result() {
@@ -1191,7 +1199,7 @@ impl CodeGenerator<'_> {
         }
         
         // end the function scope
-        // don't need to emit end scope instructions, should be handled by return
+        // don't need to emit end scope instructions, will be handled by return
         chunk.scope_mut().pop_scope();
         chunk.finish();
         
@@ -1212,7 +1220,7 @@ impl CodeGenerator<'_> {
         
         for param in signature.required.iter() {
             self.scope_mut().insert_local(param.decl, LocalName::Symbol(param.name))?;
-            self.emit_instr(None, OpCode::InsertLocal);
+            //self.emit_instr(None, OpCode::InsertLocal);
         }
         
         // will define local variables for all default arguments
@@ -1221,7 +1229,7 @@ impl CodeGenerator<'_> {
             
             for param in signature.default.iter() {
                 self.scope_mut().insert_local(param.decl, LocalName::Symbol(param.name))?;
-                self.emit_instr(symbol, OpCode::InsertLocal);
+                // self.emit_instr(symbol, OpCode::InsertLocal);
             }
         }
         
@@ -1229,7 +1237,7 @@ impl CodeGenerator<'_> {
             self.compile_variadic_arg(signature)?;
 
             self.scope_mut().insert_local(param.decl, LocalName::Symbol(param.name))?;
-            self.emit_instr(symbol, OpCode::InsertLocal);
+            // self.emit_instr(symbol, OpCode::InsertLocal);
         }
 
         Ok(())
@@ -1287,6 +1295,7 @@ impl CodeGenerator<'_> {
             let symbol = Some(param.default.debug_symbol());
             let expr = param.default.variant();
             self.compile_expr(symbol, expr)?;
+            self.emit_instr(None, OpCode::InsertLocal);
         }
         
         jump_targets.insert(default_count.into(), self.current_offset());
@@ -1329,6 +1338,7 @@ impl CodeGenerator<'_> {
         let tuple_jump_target = self.current_offset();
         self.patch_jump_instr(Jump::PopIfTrue, tuple_jump_site, Jump::PopIfTrue.dummy_width(), tuple_jump_target)?;
         self.emit_instr(None, OpCode::TupleN);
+        self.emit_instr(None, OpCode::InsertLocal);
         
         let end_jump_target = self.current_offset();
         self.patch_jump_instr(Jump::Uncond, end_jump_site, Jump::Uncond.dummy_width(), end_jump_target)?;
@@ -1336,7 +1346,7 @@ impl CodeGenerator<'_> {
         Ok(())
     }
     
-    fn compile_function_signature(&mut self, symbol: Option<&DebugSymbol>, signature: &SignatureDef) -> CompileResult<Signature> {
+    fn compile_function_signature(&mut self, symbol: Option<&DebugSymbol>, signature: &SignatureDef) -> CompileResult<UnloadedSignature> {
         let name = 
             if let Some(name) = signature.name {
                 Some(self.get_or_make_const(Constant::from(name))?)
@@ -1345,22 +1355,29 @@ impl CodeGenerator<'_> {
         let mut required = Vec::new();
         for param in signature.required.iter() {
             let name = self.get_or_make_const(Constant::from(param.name))?;
-            required.push(Parameter::new(name, param.decl));
+            required.push(UnloadedParam { name, decl: param.decl });
         }
         
         let mut default = Vec::new();
         for param in signature.default.iter() {
             let name = self.get_or_make_const(Constant::from(param.name))?;
-            default.push(Parameter::new(name, param.decl));
+            default.push(UnloadedParam { name, decl: param.decl });
         }
         
         let mut variadic = None;
         if let Some(param) = &signature.variadic {
             let name = self.get_or_make_const(Constant::from(param.name))?;
-            variadic.replace(Parameter::new(name, param.decl));
+            variadic.replace(UnloadedParam { name, decl: param.decl });
         }
         
-        Ok(Signature::new(name, required, default, variadic))
+        let signature = UnloadedSignature {
+            name,
+            required: required.into_boxed_slice(),
+            default: default.into_boxed_slice(),
+            variadic,
+        };
+        
+        Ok(signature)
     }
     
 }
