@@ -10,26 +10,28 @@ use std::mem::size_of;
 
                         // width set here so that the longest mnemonic is 16 chars
 const OP_NOP:           u8 = 0x00;
-const OP_RETURN:        u8 = 0x01;  // return from current function?
+const OP_RETURN:        u8 = 0x01;  // [ ...call frame... ret_value ] => [ ret_value ]
 
 // The odd argument order is because the VM will swap "nargs" with "arg[n]" after reading "nargs".
-// This allows the full argument list (including default and variadic arguments) to be prepared
-// in a way that can be reasoned about at compile-time.
+// This is because the function preamble will fill in any missing default arguments such that the
+// full argument list is always the same, and can be reasoned about at compile-time from the static signature.
+// This wouldn't be possible if nargs was in some arbitrary location in the middle of the argument list,
+// so swapping it to the beginning is the most efficient way to handle this.
 
-// [ callobj arg[n] arg[0] ... arg[n-1] nargs ] => [ ret_value ] 
+// [ callee arg[n] arg[0] ... arg[n-1] nargs ] => [ ret_value ] 
 const OP_CALL:          u8 = 0x02;
 
-// [ callobj arg[n] arg[0] ... arg[n-1] arg_seq nargs ] => [ ret_value ]  -- nargs does not include arg_seq! 
+// [ callee arg[n] arg[0] ... arg[n-1] arg_seq nargs ] => [ ret_value ]  -- nargs does not include arg_seq! 
 const OP_CALL_UNPACK:   u8 = 0x03;
 
 // 0x08-40        Immediate Values
 
 const OP_POP:           u8 = 0x08;  // [ _ ] => []
-const OP_DROP:          u8 = 0x09;  // (u8); [ ... ] => []
+const OP_DROP:          u8 = 0x09;  // (u8); [ value[0] ... value[N] ] => []
 const OP_CLONE:         u8 = 0x0A;  // [ value ] => [ value value ]
 
-const OP_TUPLE:         u8 = 0x0B;  // (u8); [ ... ] => [ tuple ]
-const OP_TUPLEN:        u8 = 0x0C;  // [ ... N ] => [ tuple ]
+const OP_TUPLE:         u8 = 0x0B;  // (u8); [ item[0] ... item[N] ] => [ tuple ]
+const OP_TUPLEN:        u8 = 0x0C;  // [ item[0] ... item[N] N ] => [ tuple ]
 
 const OP_LD_CONST:      u8 = 0x10;  // (u8); _ => [ value ]
 const OP_LD_CONST_16:   u8 = 0x11;  // (u16); _ => [ value ]
@@ -41,14 +43,14 @@ const OP_ST_GLOBAL:     u8 = 0x1A;  // [ value name ] => [ value ]
 const OP_LD_GLOBAL:     u8 = 0x1B;  // [ name ] => [ value ]
 const OP_DP_GLOBAL:     u8 = 0x1C;  // [ name ] => []
 
-const OP_IN_LOCAL:      u8 = 0x20;  // [ value ] => [ value ... value ]; vm.locals += 1
+const OP_IN_LOCAL:      u8 = 0x20;  // [ value ] => [ value ]; vm.locals += 1
 const OP_ST_LOCAL:      u8 = 0x21;  // (u8);  [ value ] => [ value ]
 const OP_ST_LOCAL_16:   u8 = 0x22;  // (u16); [ value ] => [ value ]
 // const OP_LD_LOCAL_32:   u8 = 0x23;  // (u32); _ => [ value ]
 const OP_LD_LOCAL:      u8 = 0x24;  // (u8);  _ => [ value ]
 const OP_LD_LOCAL_16:   u8 = 0x25;  // (u16); _ => [ value ]
 // const OP_LD_LOCAL_32:   u8 = 0x27;  // (u32); _ => [ value ]
-const OP_DP_LOCALS:     u8 = 0x26;  // (u8); [ ... ] => [] -- panics if there are any immediate values
+const OP_DP_LOCALS:     u8 = 0x26;  // (u8); [ ... local[0] ... local[N] temporaries... ] => [ temporaries... ]; vm.locals -= N
 
 // const OP_LD_NAME:       u8 = 0x28;
 // const OP_LD_INDEX:      u8 = 0x29;
@@ -323,6 +325,17 @@ impl From<OpCode> for u8 {
     fn from(opcode: OpCode) -> Self { opcode as u8 }
 }
 
+impl TryFrom<u8> for OpCode {
+    type Error = u8;
+    fn try_from(byte: u8) -> Result<Self, Self::Error> {
+        if let Some(opcode) = OpCode::from_byte(byte) {
+            Ok(opcode)
+        } else {
+            Err(byte)
+        }
+    }
+}
+
 impl PartialEq<u8> for OpCode {
     fn eq(&self, other: &u8) -> bool { *other == (*self).into() }
 }
