@@ -2,8 +2,9 @@ use std::collections::VecDeque;
 
 use log::debug;
 
+use crate::language::InternSymbol;
 use crate::lexer::{TokenMeta, Token, TokenIndex, LexerError};
-use crate::runtime::strings::{InternSymbol, StringInterner};
+use crate::runtime::strings::StringInterner;
 use crate::debug::SourceError;
 
 
@@ -154,7 +155,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                     return Some(Err(error));
                 }
                 
-                self.errors.push_back(error.with_symbol_from_ctx(&ctx));
+                self.errors.push_back(error.with_symbol_from_ctx(ctx));
                 self.synchronize_stmt(inside_block);
                 
                 // if the next token is EOF there is no point catching an error
@@ -192,10 +193,10 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             };
 
             match next.token {
-                Token::EOF | Token::Semicolon | Token::Var | 
-                Token::While  | Token::Do | Token::For | 
+                Token::EOF | Token::Semicolon |
+                Token::While  | Token::Loop | Token::For |
                 Token::Continue | Token::Break | Token::Return | 
-                Token::Echo 
+                Token::Label(..) | Token::Assert
                     => break,
                 
                 Token::End if inside_block => break,
@@ -238,10 +239,6 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             
             Token::Label(..) => self.parse_stmt_label(ctx)?,
             
-            Token::Echo => {
-                ctx.set_start(&self.advance().unwrap());
-                Stmt::Echo(self.parse_expr_variant(ctx)?)
-            },
             Token::Assert => {
                 ctx.set_start(&self.advance().unwrap());
                 Stmt::Assert(self.parse_expr_variant(ctx)?)
@@ -277,7 +274,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             Token::While => self.parse_while_loop(ctx, Some(label)),
             Token::For => unimplemented!(),
             
-            _ => return Err("labels must be followed by either a block or a loop".into()),
+            _ => Err("labels must be followed by either a block or a loop".into()),
         }
     }
     
@@ -363,8 +360,8 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                     };
                     
                     let message = format!("\"{}\" must be the last statement in a block", name);
-                    let error = ParserError::from(ErrorKind::SyntaxError(message.into()))
-                        .with_symbol_from_ctx(&ctx);
+                    let error = ParserError::from(ErrorKind::SyntaxError(message))
+                        .with_symbol_from_ctx(ctx);
                     
                     self.errors.push_back(error);
                 }
@@ -412,7 +409,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 
                 let label = self.try_parse_label(ctx)?;
                 
-                ControlFlow::Continue(label)
+                ControlFlow::Continue(label, ctx.frame().as_debug_symbol())
             },
             
             Token::Break => {
@@ -422,11 +419,11 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 let label = self.try_parse_label(ctx)?;
                 
                 let expr = 
-                    if !matches!(self.peek()?.token, Token::End | Token::Semicolon) {
+                    if !matches!(self.peek()?.token, Token::End | Token::Elif | Token::Else | Token::Semicolon ) {
                         Some(Box::new(self.parse_expr_variant(ctx)?))
                     } else { None };
                 
-                ControlFlow::Break(label, expr)
+                ControlFlow::Break(label, expr, ctx.frame().as_debug_symbol())
             },
             
             Token::Return => {
@@ -434,11 +431,11 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 ctx.set_start(&self.advance().unwrap());
                 
                 let expr = 
-                    if !matches!(self.peek()?.token, Token::End | Token::Semicolon) {
+                    if !matches!(self.peek()?.token, Token::End | Token::Elif | Token::Else | Token::Semicolon ) {
                         Some(Box::new(self.parse_expr_variant(ctx)?))
                     } else { None };
                 
-                ControlFlow::Return(expr)
+                ControlFlow::Return(expr, ctx.frame().as_debug_symbol())
             }
             
             _ => return Ok(None),
@@ -566,7 +563,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         ctx.pop_extend();
         
         let decl = Box::new(Declaration { decl, lhs, init });
-        return Ok(Expr::Declaration(decl));
+        Ok(Expr::Declaration(decl))
     }
     
     fn is_lvalue_valid_for_decl(lvalue: &LValue) -> bool {
@@ -767,6 +764,11 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             
             // Token::OpenBrace => Ok(ExprMeta::ObjectCtor(self.parse_object_constructor(ctx)?)),
             
+            Token::Echo => {
+                ctx.set_start(&self.advance().unwrap());
+                Expr::Echo(Box::new(self.parse_expr_variant(ctx)?))
+            },
+            
             _ => self.parse_primary(ctx)?,
         };
         Ok(expr)
@@ -853,7 +855,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         
         let if_expr = Expr::IfExpr { 
             branches: branches.into_boxed_slice(),
-            else_clause: else_clause.map(|expr| Box::new(expr)),
+            else_clause: else_clause.map(Box::new),
         };
         Ok(if_expr)
     }
@@ -971,7 +973,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             }
             
             ctx.push(ContextTag::FunParam);
-            ctx.set_start(&next);
+            ctx.set_start(next);
             
             // mutability modifier
             
@@ -1038,8 +1040,8 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 
                 // normal parameter
                 Token::Comma | Token::CloseParen if !is_variadic => {
-                    if default_value.is_some() {
-                        default.push(DefaultDef { name, decl, default: default_value.unwrap() });
+                    if let Some(default_expr) = default_value {
+                        default.push(DefaultDef { name, decl, default: default_expr });
                     } else {
                         if !default.is_empty() {
                             return Err("cannot have a non-default parameter after a default parameter".into());
@@ -1214,8 +1216,13 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         let mut args = Vec::new();
         let mut unpack = None;
         
+        
         // check for empty argument list
-        if !matches!(self.peek()?.token, Token::CloseParen) {
+        if matches!(self.peek()?.token, Token::CloseParen) {
+            
+            ctx.set_end(&self.advance().unwrap());
+            
+        } else {
         
             // we use the fact that argument lists parse very similarly to tuples
             // parse a single ExprMeta. If the variant is a Tuple (not a group containing a tuple!),
