@@ -11,13 +11,14 @@ use std::hash::{Hash, Hasher, BuildHasher};
 use std::collections::HashMap;
 use crate::source::ModuleSource;
 use crate::language::FloatType;
-use crate::codegen::{ProgramData, Constant, ConstID};
 use crate::runtime::{Variant, DefaultBuildHasher};
 use crate::runtime::gc::GCObject;
 use crate::runtime::strings::StringSymbol;
 use crate::runtime::types::function::Function;
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
+pub use crate::codegen::{ProgramData, Constant, ConstID, ChunkID};
+
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Access {
@@ -25,17 +26,21 @@ pub enum Access {
     ReadWrite,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Variable {
     access: Access,
     value: Variant,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Namespace {
     store: HashMap<StringSymbol, Variable, DefaultBuildHasher>,
 }
 
+impl Default for Namespace {
+    fn default() -> Self { Self::new() }
+}
+
 impl Namespace {
     pub fn new() -> Self {
         Self { 
@@ -44,26 +49,25 @@ impl Namespace {
     }
     
     // if the variable already exists, it is overwritten
-    pub fn create(&mut self, name: StringSymbol, access: Access, value: Variant) -> ExecResult<()> {
+    pub fn create(&mut self, name: StringSymbol, access: Access, value: Variant) {
         self.store.insert(name, Variable { access, value });
-        Ok(())
     }
     
     pub fn delete(&mut self, name: &StringSymbol) -> ExecResult<()> {
-        if self.store.remove(&name).is_none() {
+        if self.store.remove(name).is_none() {
             return Err(ErrorKind::NameNotDefined(name.to_string()).into())
         }
         Ok(())
     }
     
     pub fn lookup<'a>(&'a self, name: &StringSymbol) -> ExecResult<&'a Variant> {
-        self.store.get(&name)
+        self.store.get(name)
             .map(|var| &var.value)
             .ok_or_else(|| ErrorKind::NameNotDefined(name.to_string()).into())
     }
     
     pub fn lookup_mut<'a>(&'a mut self, name: &StringSymbol) -> ExecResult<&'a mut Variant> {
-        let variable = self.store.get_mut(&name)
+        let variable = self.store.get_mut(name)
             .ok_or_else(|| RuntimeError::from(ErrorKind::NameNotDefined(name.to_string())))?;
             
         if variable.access != Access::ReadWrite {
@@ -72,10 +76,16 @@ impl Namespace {
         
         Ok(&mut variable.value)
     }
+    
+    pub fn extend(&mut self, other: &Namespace) {
+        for (name, variable) in other.store.iter() {
+            self.create(*name, variable.access, variable.value.clone())
+        }
+    }
 }
 
 
-#[derive(Debug)]
+#[derive(Debug, Default, Clone)]
 pub struct GlobalEnv {
     namespace: RefCell<Namespace>,
 }
@@ -88,7 +98,7 @@ impl From<Namespace> for GlobalEnv {
 
 impl GlobalEnv {
     pub fn new() -> Self {
-        Self::from(Namespace::new())
+        Self::default()
     }
     
     pub fn borrow(&self) -> Ref<Namespace> {
@@ -135,6 +145,8 @@ impl Module {
             }
         }
     }
+    
+    
 }
 
 #[derive(Debug)]
@@ -143,6 +155,10 @@ pub struct ModuleCache {
     id_hasher: DefaultBuildHasher,
 }
 
+impl Default for ModuleCache {
+    fn default() -> Self { Self::new() }
+}
+
 impl ModuleCache {
     pub fn new() -> Self {
         Self {
@@ -155,14 +171,17 @@ impl ModuleCache {
         self.modules.get(module_id)
     }
     
-    /// Create a new module
     pub fn insert(&mut self, data: ProgramData, name: Option<String>, source: Option<ModuleSource>) -> ModuleID {
-        let module_id = self.new_module_id(name.as_ref().map(|s| s.as_str()), source.as_ref());
+        self.insert_with_globals(data, name, source, GlobalEnv::new())
+    }
+    
+    pub fn insert_with_globals(&mut self, data: ProgramData, name: Option<String>, source: Option<ModuleSource>, globals: GlobalEnv) -> ModuleID {
+        let module_id = self.new_module_id(name.as_deref(), source.as_ref());
         
         let module = Module {
             id: module_id,
             name, source, data,
-            globals: GlobalEnv::new(),
+            globals,
         };
         
         self.modules.insert(module_id, module);
