@@ -3,10 +3,10 @@ use std::rc::Rc;
 use std::hash::{Hash, Hasher};
 use std::cmp::{PartialEq, Eq};
 use crate::language::{IntType, FloatType};
-use crate::runtime::types::Metatable;
+use crate::runtime::types::{Metatable, Call};
 use crate::runtime::types::primitive::*;
 use crate::runtime::strings::{StringSymbol, STRING_TABLE};
-use crate::runtime::gc::GCHandle;
+use crate::runtime::gc::{GCHandle, GCObject};
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
 
@@ -72,6 +72,19 @@ impl Variant {
         }
         
     }
+    
+    pub fn invoke(&self, args: &[Variant]) -> ExecResult<Call> {
+        // TODO make GC objects suck less
+        if let Self::Object(handle) = self {
+            return handle.with_ref(|obj| {
+                let GCObject::Function(fun) = &*obj;
+                fun.signature().check_args(args)?;
+                Ok(fun.as_call())
+            })
+        }
+        
+        Err(ErrorKind::NotCallable(self.clone()).into())
+    }
 }
 
 impl From<bool> for Variant {
@@ -118,7 +131,7 @@ impl<'a> TryFrom<&'a Variant> for VariantKey<'a> {
         if !value.can_hash() {
             return Err(ErrorKind::UnhashableValue(value.clone()).into());
         }
-        Ok(Self(&value))
+        Ok(Self(value))
     }
 }
 
@@ -221,3 +234,4 @@ impl fmt::Display for Variant {
 
 
 
+
