@@ -2,25 +2,38 @@ use crate::language::{IntType, FloatType};
 use crate::codegen::{Program, ProgramData, ChunkID, ConstID, Constant, OpCode};
 use crate::runtime::Variant;
 use crate::runtime::ops;
+use crate::runtime::types::Call;
 use crate::runtime::strings::StringSymbol;
 use crate::runtime::module::{Module, ModuleCache, ModuleID, Access, GlobalEnv};
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
+use crate::debug::traceback::CallSite;
+use crate::debug::snapshot::{VMSnapshot, VMStateSnapshot};
 
 
+// data used to set up a new call
+struct CallInfo {
+    frame: usize,
+    call: Call,
+    site: CallSite,
+}
+
 enum Control {
     Continue,
+    Call(CallInfo),
     Return,
     Exit,
 }
 
+
 // Stack-based Virtual Machine
 #[derive(Debug)]
 pub struct VirtualMachine<'m> {
     module_cache: &'m ModuleCache,
-    repl_env: Option<&'m GlobalEnv>,
+    traceback: Vec<CallSite>,
     
-    states: Vec<VMState<'m>>,
+    calls: Vec<VMState<'m>>,
     values: ValueStack,
+    state: VMState<'m>,
 }
 
 impl<'m> VirtualMachine<'m> {
@@ -30,131 +43,93 @@ impl<'m> VirtualMachine<'m> {
         
         Self {
             module_cache,
-            repl_env: None,
+            traceback: Vec::new(),
+            calls: Vec::new(),
             values: ValueStack::new(),
-            states: vec![ VMState::new(module, main_chunk, 0) ],
+            state: VMState::main_chunk(module, main_chunk),
         }
     }
     
-    pub fn new_repl(module_cache: &'m ModuleCache, repl_env: &'m GlobalEnv, module_id: ModuleID, main_chunk: &'m [u8]) -> Self {
+    pub fn repl(module_cache: &'m ModuleCache, repl_env: &'m GlobalEnv, module_id: ModuleID, main_chunk: &'m [u8]) -> Self {
         let module = module_cache.get(&module_id).expect("invalid module id");
         
         Self {
             module_cache,
-            repl_env: Some(repl_env),
+            traceback: Vec::new(),
+            calls: Vec::new(),
             values: ValueStack::new(),
-            states: vec![ VMState::with_globals(module, repl_env, main_chunk, 0) ],
+            state: VMState::main_repl(module, repl_env, main_chunk),
         }
     }
     
     pub fn run(mut self) -> ExecResult<()> {
-        while let Some(state) = self.states.last_mut() {
-            match state.exec_next(&mut self.values)? {
-                Control::Continue => { }
-                Control::Return | Control::Exit => break,
-            }
-        }
+        while !self.exec_next()? { }
         Ok(())
     }
+    
+    pub fn run_steps(self) -> impl Iterator<Item=ExecResult<VMSnapshot>> + 'm {
+        VMStepper::from(self)
+    }
+    
+    #[inline]
+    fn exec_next(&mut self) -> ExecResult<bool> {
+        match self.state.exec_next(&mut self.values)? {
+            Control::Exit => return Ok(true),
+            Control::Return if self.calls.is_empty() => return Ok(true),
+            
+            Control::Call(info) => self.setup_call(info)?,
+            Control::Return => self.return_call(),
+            Control::Continue => { }
+        }
+        Ok(false)
+    }
+    
+    fn setup_call(&mut self, call: CallInfo) -> ExecResult<()> {
+        self.traceback.push(call.site);
+        
+        match call.call {
+            Call::Native(retval) => {
+                let retval = retval?;
+                self.values.truncate(call.frame.into());
+                self.values.push(retval);
+                self.traceback.pop();
+            },
+            
+            Call::Chunk(module_id, chunk_id) => {
+                let module = self.module_cache.get(&module_id)
+                    .expect("invalid module id");
+                
+                let locals = LocalIndex::try_from(self.values.len() - call.frame)
+                    .expect("local index overflow");
+                
+                let mut state = VMState::call_frame(module, chunk_id, call.frame, locals);
+                std::mem::swap(&mut self.state, &mut state);
+                self.calls.push(state);
+                
+                log::debug!("Setup call: {{ frame: {}, locals: {} }}", self.state.frame, self.state.locals);
+                log::debug!("Stack: {:?}", self.values);
+            },
+        }
+        
+        Ok(())
+    }
+    
+    fn return_call(&mut self) {
+        let frame = self.state.frame;
+        
+        let mut state = self.calls.pop().expect("empty call stack");
+        std::mem::swap(&mut self.state, &mut state);
+        
+        let retval = self.values.pop();
+        self.values.truncate(frame.into());
+        self.values.push(retval);
+        self.traceback.pop();
+        
+        log::debug!("Return call: {{ frame: {}, locals: {} }}", self.state.frame, self.state.locals);
+        log::debug!("Stack: {:?}", self.values);
+    }
 }
 
-// Stack Manipulation
-#[derive(Debug)]
-struct ValueStack {
-    stack: Vec<Variant>,
-}
-
-impl ValueStack {
-    fn new() -> Self {
-        Self { stack: Vec::new() }
-    }
-    
-    fn take(self) -> Vec<Variant> {
-        self.stack
-    }
-    
-    // Note: when moving values off the stack, make sure to copy *before* popping
-    // This ensures that the GC sees the values as rooted
-    
-    #[inline(always)]
-    fn len(&self) -> usize {
-        self.stack.len()
-    }
-    
-    #[inline(always)]
-    fn clear(&mut self) {
-        self.stack.clear()
-    }
-    
-    #[inline(always)]
-    fn pop(&mut self) -> Variant {
-        self.stack.pop().expect("empty stack")
-    }
-    
-    #[inline(always)]
-    fn pop_many(&mut self, count: usize) -> Vec<Variant> {
-        self.stack.split_off(self.stack.len() - count)
-    }
-    
-    #[inline(always)]
-    fn discard(&mut self, count: usize) {
-        self.stack.truncate(self.stack.len() - count)
-    }
-    
-    #[inline(always)]
-    fn discard_at(&mut self, index: usize, count: usize) {
-        let discard_range = index..(index + count);
-        self.stack.splice(discard_range, std::iter::empty());
-    }
-    
-    #[inline(always)]
-    fn push(&mut self, value: Variant) {
-        self.stack.push(value)
-    }
-    
-    #[inline(always)]
-    fn insert(&mut self, index: usize, value: Variant) {
-        self.stack.insert(index, value);
-    }
-    
-    #[inline(always)]
-    fn replace(&mut self, value: Variant) {
-        *self.stack.last_mut().expect("empty stack") = value;
-    }
-    
-    #[inline(always)]
-    fn replace_many(&mut self, count: usize, value: Variant) {
-        let replace_range = (self.stack.len()-count).. ;
-        self.stack.splice(replace_range, std::iter::once(value));
-    }
-    
-    #[inline(always)]
-    fn replace_at(&mut self, index: usize, value: Variant) {
-        let item = self.stack.get_mut(index)
-            .expect("value index out of bounds");
-        *item = value;
-    }
-    
-    #[inline(always)]
-    fn peek(&self) -> &Variant {
-        self.stack.last().expect("empty stack")
-    }
-    
-    #[inline(always)]
-    fn peek_at(&self, index: usize) -> &Variant {
-        self.stack.get(index)
-            .expect("value index out of bounds")
-    }
-    
-    #[inline(always)]
-    fn peek_many(&self, count: usize) -> &[Variant] {
-        let (_, peek) = self.stack.as_slice().split_at(self.stack.len() - count);
-        peek
-    }
-    
-}
-
-
 // Helper macros
 macro_rules! read_le_bytes {
     ( $type:ty, $data:expr ) => {
@@ -205,7 +180,7 @@ macro_rules! cond_jump {
 }
 
 
-pub type LocalIndex = u16;
+pub type LocalIndex = u16;  // TODO make this u32 and add opcodes
 
 // stores the active chunk execution information
 #[derive(Debug)]
@@ -213,26 +188,56 @@ struct VMState<'m> {
     module: &'m Module,
     globals: &'m GlobalEnv,
     chunk: &'m [u8],
-    frame: usize,  // index of the first local
+    chunk_id: Option<ChunkID>,
+    frame: usize,
     locals: LocalIndex,
     pc: usize,
 }
 
 impl<'m> VMState<'m> {
-    fn new(module: &'m Module, chunk: &'m [u8], frame: usize) -> Self {
-        Self::with_globals(module, module.globals(), chunk, frame)
+    fn call_frame(module: &'m Module, chunk_id: ChunkID, frame: usize, locals: LocalIndex) -> Self {
+        Self {
+            module,
+            globals: module.globals(),
+            chunk: module.data().get_chunk(chunk_id),
+            chunk_id: Some(chunk_id),
+            locals,
+            frame,
+            pc: 0,
+        }
     }
     
-    fn with_globals(module: &'m Module, globals: &'m GlobalEnv, chunk: &'m [u8], frame: usize) -> Self {
+    fn main_chunk(module: &'m Module, chunk: &'m [u8]) -> Self {
+        Self {
+            module,
+            globals: module.globals(),
+            chunk,
+            chunk_id: None,
+            locals: 0,
+            frame: 0,
+            pc: 0,
+        }
+    }
+    
+    fn main_repl(module: &'m Module, globals: &'m GlobalEnv, chunk: &'m [u8]) -> Self {
         Self {
             module,
             globals,
             chunk,
-            frame,
+            chunk_id: None,
             locals: 0,
+            frame: 0,
             pc: 0,
         }
     }
+    
+    fn get_callsite(&self, offset: usize) -> CallSite {
+        CallSite::Chunk {
+            offset,
+            module_id: self.module.module_id(),
+            chunk_id: self.chunk_id,
+        }
+    }
 
     #[inline(always)]
     fn offset_pc(&self, offset: isize) -> Option<usize> {
@@ -268,6 +273,7 @@ impl<'m> VMState<'m> {
             .unwrap_or_else(|| panic!("invalid instruction: {:x}", op_byte));
         
         let data_slice = (self.pc + 1) .. (self.pc + opcode.instr_len());
+        let current_offset = self.pc;
         self.pc += opcode.instr_len(); // pc points to next instruction
         
         let data = self.chunk.get(data_slice).expect("truncated instruction");
@@ -276,7 +282,22 @@ impl<'m> VMState<'m> {
             
             OpCode::Return => return Ok(Control::Return),
             
-            OpCode::Call => unimplemented!(),
+            OpCode::Call => {
+                let nargs = Self::into_usize(stack.peek().clone());
+                stack.swap_last(stack.len() - nargs - 1);
+                
+                let frame = stack.len() - nargs - 2;
+                let callee = stack.peek_at(frame);
+                
+                let args = stack.peek_many(nargs);
+                let call = CallInfo {
+                    call: callee.invoke(args)?,
+                    frame: frame.try_into().expect("frame index overflow"),
+                    site: self.get_callsite(current_offset),
+                };
+                return Ok(Control::Call(call))
+            },
+            
             OpCode::CallUnpack => unimplemented!(),
             
             OpCode::Pop => { 
@@ -304,12 +325,12 @@ impl<'m> VMState<'m> {
             OpCode::InsertGlobal => {
                 let name = Self::into_name(stack.pop());
                 let value = stack.peek().clone();
-                self.globals.borrow_mut().create(name, Access::ReadOnly, value)?;
+                self.globals.borrow_mut().create(name, Access::ReadOnly, value);
             },
             OpCode::InsertGlobalMut => {
                 let name = Self::into_name(stack.pop());
                 let value = stack.peek().clone();
-                self.globals.borrow_mut().create(name, Access::ReadWrite, value)?;
+                self.globals.borrow_mut().create(name, Access::ReadWrite, value);
             },
             OpCode::StoreGlobal => {
                 let name = Self::into_name(stack.pop());
@@ -334,28 +355,29 @@ impl<'m> VMState<'m> {
             },
             OpCode::StoreLocal => {
                 let index = usize::from(data[0]);
-                stack.replace_at(index, stack.peek().clone());
+                stack.replace_at(self.frame + index, stack.peek().clone());
             },
             OpCode::StoreLocal16 => {
                 let index = usize::from(read_le_bytes!(u16, data));
-                stack.replace_at(index, stack.peek().clone());
+                stack.replace_at(self.frame + index, stack.peek().clone());
             },
             OpCode::LoadLocal => {
                 let index = usize::from(data[0]);
                 debug_assert!(index < self.locals.into());
-                stack.push(stack.peek_at(index).clone());
+                stack.push(stack.peek_at(self.frame + index).clone());
             },
             OpCode::LoadLocal16 => {
                 let index = usize::from(read_le_bytes!(u16, data));
                 debug_assert!(index < self.locals.into());
-                stack.push(stack.peek_at(index).clone());
+                stack.push(stack.peek_at(self.frame + index).clone());
             },
             OpCode::DropLocals => {
                 let count = LocalIndex::from(data[0]);
-                if stack.len() == self.locals.into() {
+                let locals = self.frame + usize::from(self.locals);
+                if stack.len() == locals {
                     stack.discard(usize::from(count));
                 } else {
-                    let index = usize::from(self.locals) - usize::from(count);
+                    let index = usize::from(locals) - usize::from(count);
                     stack.discard_at(index, usize::from(count));
                 }
                 self.locals -= count;
@@ -365,14 +387,13 @@ impl<'m> VMState<'m> {
             OpCode::True => stack.push(Variant::BoolTrue),
             OpCode::False => stack.push(Variant::BoolFalse),
             OpCode::Empty => stack.push(Variant::EmptyTuple),
-
+            
             OpCode::Tuple => {
                 let tuple_len = usize::from(data[0]);
                 
                 // peek and clone to ensure items stay rooted for GC
-                let items = stack.peek_many(tuple_len).iter()
-                    .cloned().collect::<Vec<Variant>>()
-                    .into_boxed_slice();
+                let items = stack.peek_many(tuple_len)
+                    .to_vec().into_boxed_slice();
                 
                 let tuple = Variant::make_tuple(items);
                 stack.replace_many(tuple_len, tuple);
@@ -381,9 +402,8 @@ impl<'m> VMState<'m> {
                 let tuple_len = Self::into_usize(stack.pop());
                 
                 // peek and clone to ensure items stay rooted for GC
-                let items = stack.peek_many(tuple_len).iter()
-                    .cloned().collect::<Vec<Variant>>()
-                    .into_boxed_slice();
+                let items = stack.peek_many(tuple_len)
+                    .to_vec().into_boxed_slice();
                 
                 let tuple = Variant::make_tuple(items);
                 stack.replace_many(tuple_len, tuple);
@@ -455,3 +475,177 @@ impl<'m> VMState<'m> {
         Ok(Control::Continue)
     }
 }
+
+
+// Stack Manipulation
+#[derive(Debug)]
+struct ValueStack {
+    stack: Vec<Variant>,
+}
+
+impl ValueStack {
+    fn new() -> Self {
+        Self { stack: Vec::new() }
+    }
+    
+    fn take(self) -> Vec<Variant> {
+        self.stack
+    }
+    
+    // Note: when moving values off the stack, make sure to copy *before* popping
+    // This ensures that the GC sees the values as rooted
+    
+    #[inline(always)]
+    fn len(&self) -> usize {
+        self.stack.len()
+    }
+    
+    #[inline(always)]
+    fn clear(&mut self) {
+        self.stack.clear()
+    }
+    
+    #[inline(always)]
+    fn pop(&mut self) -> Variant {
+        self.stack.pop().expect("empty stack")
+    }
+    
+    #[inline(always)]
+    fn pop_many(&mut self, count: usize) -> Vec<Variant> {
+        self.stack.split_off(self.stack.len() - count)
+    }
+
+    #[inline(always)]
+    fn truncate(&mut self, index: usize) {
+        self.stack.truncate(index)
+    }
+    
+    #[inline(always)]
+    fn discard(&mut self, count: usize) {
+        self.stack.truncate(self.stack.len() - count)
+    }
+    
+    #[inline(always)]
+    fn discard_at(&mut self, index: usize, count: usize) {
+        let discard_range = index..(index + count);
+        self.stack.splice(discard_range, std::iter::empty());
+    }
+    
+    #[inline(always)]
+    fn push(&mut self, value: Variant) {
+        self.stack.push(value)
+    }
+    
+    #[inline(always)]
+    fn insert(&mut self, index: usize, value: Variant) {
+        self.stack.insert(index, value);
+    }
+    
+    #[inline(always)]
+    fn replace(&mut self, value: Variant) {
+        *self.stack.last_mut().expect("empty stack") = value;
+    }
+    
+    #[inline(always)]
+    fn replace_many(&mut self, count: usize, value: Variant) {
+        let replace_range = (self.stack.len()-count).. ;
+        self.stack.splice(replace_range, std::iter::once(value));
+    }
+    
+    #[inline(always)]
+    fn replace_at(&mut self, index: usize, value: Variant) {
+        let item = self.stack.get_mut(index)
+            .expect("value index out of bounds");
+        *item = value;
+    }
+    
+    #[inline(always)]
+    fn swap_last(&mut self, index: usize) {
+        let len = self.stack.len();
+        self.stack.swap(index, len - 1)
+    }
+    
+    #[inline(always)]
+    fn peek(&self) -> &Variant {
+        self.stack.last().expect("empty stack")
+    }
+    
+    #[inline(always)]
+    fn peek_at(&self, index: usize) -> &Variant {
+        self.stack.get(index)
+            .expect("value index out of bounds")
+    }
+    
+    #[inline(always)]
+    fn peek_many(&self, count: usize) -> &[Variant] {
+        let (_, peek) = self.stack.as_slice().split_at(self.stack.len() - count);
+        peek
+    }
+    
+}
+
+
+// For debugging
+
+
+
+impl From<&VirtualMachine<'_>> for VMSnapshot {
+    fn from (vm: &VirtualMachine) -> Self {
+        Self {
+            calls: vm.calls.iter().map(VMStateSnapshot::from).collect(),
+            values: vm.values.stack.clone(),
+            state: (&vm.state).into(),
+        }
+    }
+}
+
+
+
+impl From<&VMState<'_>> for VMStateSnapshot {
+    fn from(state: &VMState) -> Self {
+        let next_instr = state.chunk.get(state.pc)
+            .map(|byte| OpCode::try_from(*byte).map_or_else(
+                |byte| vec![ byte ],
+                |opcode| state.chunk[state.pc..(state.pc + opcode.instr_len())].to_vec()
+            ));
+        
+        Self {
+            module_id: state.module.module_id(),
+            chunk_id: state.chunk_id,
+            frame: state.frame,
+            locals: state.locals,
+            pc: state.pc,
+            next_instr,
+        }
+    }
+}
+
+struct VMStepper<'m> {
+    vm: VirtualMachine<'m>,
+    stop: bool,
+}
+
+impl<'m> From<VirtualMachine<'m>> for VMStepper<'m> {
+    fn from(vm: VirtualMachine<'m>) -> Self {
+        Self { vm, stop: false }
+    }
+}
+
+impl<'m> Iterator for VMStepper<'m> {
+    type Item = ExecResult<VMSnapshot>;
+    
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.stop {
+            return None
+        }
+        
+        let status = self.vm.exec_next();
+        if let Err(error) = status {
+            self.stop = true;
+            return Some(Err(error));
+        }
+        
+        self.stop = status.unwrap();
+        Some(Ok(VMSnapshot::from(&self.vm)))
+    }
+}
\ No newline at end of file
